@@ -34,7 +34,7 @@
  *
  * 30.10.94  -  Added support for v2 filesystem
  *		(Andreas Schwab, schwab@issan.informatik.uni-dortmund.de)
- * 
+ *
  * 09.11.94  -	Added test to prevent overwrite of mounted fs adapted
  *		from Theodore Ts'o's (tytso@athena.mit.edu) mke2fs
  *		program.  (Daniel Quinlan, quinlan@yggdrasil.com)
@@ -77,6 +77,8 @@
 #include "all-io.h"
 #include "closestream.h"
 #include "ismounted.h"
+
+#define XALLOC_EXIT_CODE MKFS_EX_ERROR
 #include "xalloc.h"
 
 #define MINIX_ROOT_INO 1
@@ -103,6 +105,7 @@ static char *inode_buffer = NULL;
 struct fs_control {
 	char *device_name;		/* device on a Minix file system is created */
 	int device_fd;			/* open file descriptor of the device */
+	char *lockmode;			/* as specified by --lock */
 	unsigned long long fs_blocks;	/* device block count for the file system */
 	int fs_used_blocks;		/* used blocks on a device */
 	int fs_bad_blocks;		/* number of bad blocks found from device */
@@ -129,8 +132,9 @@ static char *zone_map;
 #define mark_zone(x) (setbit(zone_map,(x)-get_first_zone()+1))
 #define unmark_zone(x) (clrbit(zone_map,(x)-get_first_zone()+1))
 
-static void __attribute__((__noreturn__)) usage(FILE *out)
+static void __attribute__((__noreturn__)) usage(void)
 {
+	FILE *out = stdout;
 	fputs(USAGE_HEADER, out);
 	fprintf(out, _(" %s [options] /dev/name [blocks]\n"), program_invocation_short_name);
 	fputs(USAGE_OPTIONS, out);
@@ -141,20 +145,21 @@ static void __attribute__((__noreturn__)) usage(FILE *out)
 	fputs(_(" -i, --inodes <num>      number of inodes for the filesystem\n"), out);
 	fputs(_(" -c, --check             check the device for bad blocks\n"), out);
 	fputs(_(" -l, --badblocks <file>  list of bad blocks from file\n"), out);
+	fprintf(out, _(
+		"     --lock[=<mode>]     use exclusive device lock (%s, %s or %s)\n"), "yes", "no", "nonblock");
 	fputs(USAGE_SEPARATOR, out);
-	fputs(USAGE_HELP, out);
-	fputs(USAGE_VERSION, out);
-	fprintf(out, USAGE_MAN_TAIL("mkfs.minix(8)"));
-	exit(out == stderr ? MKFS_EX_USAGE : MKFS_EX_OK);
+	printf(USAGE_HELP_OPTIONS(25));
+	printf(USAGE_MAN_TAIL("mkfs.minix(8)"));
+	exit(MKFS_EX_OK);
 }
 
 #ifdef TEST_SCRIPT
 static inline time_t mkfs_minix_time(time_t *t)
 {
 	const char *str = getenv("MKFS_MINIX_TEST_SECOND_SINCE_EPOCH");
-	time_t sec;
+	uint64_t sec;
 
-	if (str && sscanf(str, "%ld", &sec) == 1)
+	if (str && sscanf(str, "%"SCNd64, &sec) == 1)
 		return sec;
 	return time(t);
 }
@@ -534,9 +539,9 @@ static void setup_tables(const struct fs_control *ctl) {
 	if (fs_version == 3)
 		Super3.s_ninodes = inodes;
 	else {
-		Super.s_ninodes = inodes;
 		if (inodes > MINIX_MAX_INODES)
 			inodes = MINIX_MAX_INODES;
+		Super.s_ninodes = inodes;
 	}
 	super_set_map_blocks(ctl, inodes);
 	if (MINIX_MAX_INODES < first_zone_data())
@@ -575,7 +580,7 @@ static void setup_tables(const struct fs_control *ctl) {
  */
 static size_t do_check(const struct fs_control *ctl, char * buffer, int try, unsigned int current_block) {
 	ssize_t got;
-	
+
 	/* Seek to the correct loc. */
 	if (lseek(ctl->device_fd, current_block * MINIX_BLOCK_SIZE, SEEK_SET) !=
 		       current_block * MINIX_BLOCK_SIZE )
@@ -584,7 +589,7 @@ static size_t do_check(const struct fs_control *ctl, char * buffer, int try, uns
 
 	/* Try the read */
 	got = read(ctl->device_fd, buffer, try * MINIX_BLOCK_SIZE);
-	if (got < 0) got = 0;	
+	if (got < 0) got = 0;
 	if (got & (MINIX_BLOCK_SIZE - 1 )) {
 		printf(_("Weird values in do_check: probably bugs\n"));
 	}
@@ -683,7 +688,7 @@ static int find_super_magic(const struct fs_control *ctl)
 
 static void determine_device_blocks(struct fs_control *ctl, const struct stat *statbuf)
 {
-	unsigned long long dev_blocks;
+	unsigned long long dev_blocks = 0;
 
 	if (S_ISBLK(statbuf->st_mode)) {
 		int sectorsize;
@@ -743,6 +748,9 @@ int main(int argc, char ** argv)
 	int i;
 	struct stat statbuf;
 	char * listfile = NULL;
+	enum {
+		OPT_LOCK = CHAR_MAX + 1
+	};
 	static const struct option longopts[] = {
 		{"namelength", required_argument, NULL, 'n'},
 		{"inodes", required_argument, NULL, 'i'},
@@ -750,13 +758,16 @@ int main(int argc, char ** argv)
 		{"badblocks", required_argument, NULL, 'l'},
 		{"version", no_argument, NULL, 'V'},
 		{"help", no_argument, NULL, 'h'},
+		{"lock",optional_argument, NULL, OPT_LOCK},
 		{NULL, 0, NULL, 0}
 	};
 
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
-	atexit(close_stdout);
+	close_stdout_atexit();
+
+	strutils_set_exitcode(MKFS_EX_USAGE);
 
 	while ((i = getopt_long(argc, argv, "1v23n:i:cl:Vh", longopts, NULL)) != -1)
 		switch (i) {
@@ -787,13 +798,20 @@ int main(int argc, char ** argv)
 		case 'l':
 			listfile = optarg;
 			break;
+		case OPT_LOCK:
+			ctl.lockmode = "1";
+			if (optarg) {
+				if (*optarg == '=')
+					optarg++;
+				ctl.lockmode = optarg;
+			}
+			break;
 		case 'V':
-			printf(UTIL_LINUX_VERSION);
-			return MKFS_EX_OK;
+			print_version(MKFS_EX_OK);
 		case 'h':
-			usage(stdout);
+			usage();
 		default:
-			usage(stderr);
+			errtryhelp(MKFS_EX_USAGE);
 		}
 	argc -= optind;
 	argv += optind;
@@ -806,7 +824,8 @@ int main(int argc, char ** argv)
 		ctl.fs_blocks = strtoul_or_err(argv[0], _("failed to parse number of blocks"));
 
 	if (!ctl.device_name) {
-		usage(stderr);
+		warnx(_("no device specified"));
+		errtryhelp(MKFS_EX_USAGE);
 	}
 	check_user_instructions(&ctl);
 	if (is_mounted(ctl.device_name))
@@ -817,6 +836,8 @@ int main(int argc, char ** argv)
 	ctl.device_fd = open_blkdev_or_file(&statbuf, ctl.device_name, O_RDWR);
 	if (ctl.device_fd < 0)
 		err(MKFS_EX_ERROR, _("cannot open %s"), ctl.device_name);
+	if (blkdev_lock(ctl.device_fd, ctl.device_name, ctl.lockmode) != 0)
+		exit(MKFS_EX_ERROR);
 	determine_device_blocks(&ctl, &statbuf);
 	setup_tables(&ctl);
 	if (ctl.check_blocks)
