@@ -338,6 +338,7 @@ int mnt_optlist_merge_opts(struct libmnt_optlist *ls)
 
 			/* remove inverted option */
 			else if (opt->ent && x->ent
+			    && opt->map == x->map
 			    && opt->ent->id == x->ent->id
 			    && (opt->ent->mask & MNT_INVERT
 				    || x->ent->mask & MNT_INVERT))
@@ -462,7 +463,7 @@ static struct libmnt_opt *optlist_new_opt(struct libmnt_optlist *ls,
 		else if (opt->ent->id == MS_BIND)
 			ls->is_bind = 1;
 		else if (opt->ent->id == MS_RDONLY)
-			ls->is_rdonly = 1;
+			ls->is_rdonly = opt->ent->mask & MNT_INVERT ? 0 : 1;
 		else if (opt->ent->id == MS_MOVE)
 			ls->is_move = 1;
 		else if (opt->ent->id == MS_SILENT)
@@ -520,6 +521,8 @@ static int optlist_add_optstr(struct libmnt_optlist *ls, const char *optstr,
 		if (!opt)
 			return -ENOMEM;
 		opt->src = MNT_OPTSRC_STRING;
+		if (where)
+			where = &opt->opts;
 	}
 
 	optlist_cleanup_cache(ls);
@@ -622,6 +625,8 @@ static int optlist_add_flags(struct libmnt_optlist *ls, unsigned long flags,
 		if (!opt)
 			return -ENOMEM;
 		opt->src = MNT_OPTSRC_FLAG;
+		if (where)
+			where = &opt->opts;
 	}
 
 	optlist_cleanup_cache(ls);
