@@ -1,10 +1,13 @@
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 /*
- * version.c - Return the version of the libmount library
+ * This file is part of libmount from util-linux project.
  *
- * Copyright (C) 2008 Karel Zak <kzak@redhat.com>
- * [Based on libblkid/version.c by Theodore Ts'o]
+ * Copyright (C) 2008-2018 Karel Zak <kzak@redhat.com>
  *
- * See COPYING.libmount for the License of this software.
+ * libmount is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
+ * (at your option) any later version.
  */
 
 /**
@@ -16,6 +19,7 @@
 #include <ctype.h>
 
 #include "mountP.h"
+#include "mount-api-utils.h"
 
 static const char *lib_version = LIBMOUNT_VERSION;
 static const char *lib_features[] = {
@@ -28,8 +32,17 @@ static const char *lib_features[] = {
 #ifdef HAVE_BTRFS_SUPPORT
 	"btrfs",
 #endif
-#ifdef USE_LIBMOUNT_FORCE_MOUNTINFO
-	"force-mountinfo",
+#ifdef HAVE_CRYPTSETUP
+	"verity",
+#endif
+#ifdef USE_LIBMOUNT_SUPPORT_NAMESPACES
+	"namespaces",
+#endif
+#if defined(HAVE_MOUNTFD_API) && defined(HAVE_LINUX_MOUNT_H)
+	"idmapping",
+#endif
+#ifdef USE_LIBMOUNT_MOUNTFD_SUPPORT
+	"fd-based-mount",
 #endif
 #if !defined(NDEBUG)
 	"assert",	/* libc assert.h stuff */
@@ -86,7 +99,7 @@ int mnt_get_library_version(const char **ver_string)
  * Example:
  * <informalexample>
  *   <programlisting>
- *	const char *features;
+ *	const char **features;
  *
  *	mnt_get_library_features(&features);
  *	while (features && *features)
@@ -110,6 +123,10 @@ static int test_version(struct libmnt_test *ts, int argc, char *argv[])
 	const char *ver;
 	const char **features;
 
+	if (argc == 2)
+		printf("Your version: %d\n",
+				mnt_parse_version_string(argv[1]));
+
 	mnt_get_library_version(&ver);
 
 	printf("Library version: %s\n", ver);
@@ -120,6 +137,8 @@ static int test_version(struct libmnt_test *ts, int argc, char *argv[])
 	while (features && *features)
 		printf(" %s", *features++);
 
+	printf("\n");
+
 	if (mnt_get_library_version(NULL) ==
 			mnt_parse_version_string(LIBMOUNT_VERSION))
 		return 0;
