@@ -54,28 +54,33 @@ static const char *idtype[] = {
 	[PRIO_USER]	= N_("user ID"),
 };
 
-static void __attribute__((__noreturn__)) usage(FILE *out)
+static void __attribute__((__noreturn__)) usage(void)
 {
+	FILE *out = stdout;
 	fputs(USAGE_HEADER, out);
 	fprintf(out,
-	      _(" %1$s [-n] <priority> [-p|--pid] <pid>...\n"
-		" %1$s [-n] <priority>  -g|--pgrp <pgid>...\n"
-		" %1$s [-n] <priority>  -u|--user <user>...\n"),
+	      _(" %1$s [-n|--priority|--relative] <priority> [-p|--pid] <pid>...\n"
+		" %1$s [-n|--priority|--relative] <priority>  -g|--pgrp <pgid>...\n"
+		" %1$s [-n|--priority|--relative] <priority>  -u|--user <user>...\n"),
 		program_invocation_short_name);
 
 	fputs(USAGE_SEPARATOR, out);
 	fputs(_("Alter the priority of running processes.\n"), out);
 
 	fputs(USAGE_OPTIONS, out);
-	fputs(_(" -n, --priority <num>   specify the nice increment value\n"), out);
-	fputs(_(" -p, --pid <id>         interpret argument as process ID (default)\n"), out);
-	fputs(_(" -g, --pgrp <id>        interpret argument as process group ID\n"), out);
-	fputs(_(" -u, --user <name>|<id> interpret argument as username or user ID\n"), out);
+	fputs(_(" -n <num>               specify the nice value\n"), out);
+	fputs(_("                          If POSIXLY_CORRECT flag is set in environment\n"), out);
+	fputs(_("                          then the priority is 'relative' to current\n"), out);
+	fputs(_("                          process priority. Otherwise it is 'absolute'.\n"), out);
+	fputs(_(" --priority <num>       specify the 'absolute' nice value\n"), out);
+	fputs(_(" --relative <num>       specify the 'relative' nice value\n"), out);
+	fputs(_(" -p, --pid              interpret arguments as process ID (default)\n"), out);
+	fputs(_(" -g, --pgrp             interpret arguments as process group ID\n"), out);
+	fputs(_(" -u, --user             interpret arguments as username or user ID\n"), out);
 	fputs(USAGE_SEPARATOR, out);
-	fputs(USAGE_HELP, out);
-	fputs(USAGE_VERSION, out);
-	fprintf(out, USAGE_MAN_TAIL("renice(1)"));
-	exit(out == stderr ? EXIT_FAILURE : EXIT_SUCCESS);
+	printf(USAGE_HELP_OPTIONS(24));
+	printf(USAGE_MAN_TAIL("renice(1)"));
+	exit(EXIT_SUCCESS);
 }
 
 static int getprio(const int which, const int who, int *prio)
@@ -89,13 +94,19 @@ static int getprio(const int which, const int who, int *prio)
 	return 0;
 }
 
-static int donice(const int which, const int who, const int prio)
+static int donice(const int which, const int who, const int prio, const int relative)
 {
 	int oldprio, newprio;
 
 	if (getprio(which, who, &oldprio) != 0)
 		return 1;
-	if (setpriority(which, who, prio) < 0) {
+	
+	newprio = prio; // if not relative, set absolute priority
+
+	if (relative)
+		newprio = oldprio + prio;
+
+	if (setpriority(which, who, newprio) < 0) {
 		warn(_("failed to set priority for %d (%s)"), who, idtype[which]);
 		return 1;
 	}
@@ -114,12 +125,13 @@ int main(int argc, char **argv)
 {
 	int which = PRIO_PROCESS;
 	int who = 0, prio, errs = 0;
+	int relative = 0;
 	char *endptr = NULL;
 
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
-	atexit(close_stdout);
+	close_stdout_atexit();
 
 	argc--;
 	argv++;
@@ -127,28 +139,48 @@ int main(int argc, char **argv)
 	if (argc == 1) {
 		if (strcmp(*argv, "-h") == 0 ||
 		    strcmp(*argv, "--help") == 0)
-			usage(stdout);
+			usage();
 
 		if (strcmp(*argv, "-v") == 0 ||
 		    strcmp(*argv, "-V") == 0 ||
-		    strcmp(*argv, "--version") == 0) {
-			printf(UTIL_LINUX_VERSION);
-			return EXIT_SUCCESS;
+		    strcmp(*argv, "--version") == 0)
+			print_version(EXIT_SUCCESS);
+	}
+
+	if (*argv)
+	{
+		if (strcmp(*argv, "-n") == 0)
+		{
+			// Fully conform to posix only if POSIXLY_CORRECT is
+			// set in the environment. If not, use the absolute
+			// value as it's been used (incorrectly) since 2009
+			relative = getenv("POSIXLY_CORRECT") == NULL ? 0 : 1;
+			argc--;
+			argv++;
+		}
+		else if (strcmp(*argv, "--relative") == 0)
+		{
+			relative = 1;
+			argc--;
+			argv++;
+		}
+		else if (strcmp(*argv, "--priority") == 0) {
+			relative = 0;
+			argc--;
+			argv++;
 		}
 	}
 
-	if (*argv && (strcmp(*argv, "-n") == 0 || strcmp(*argv, "--priority") == 0)) {
-		argc--;
-		argv++;
+	if (argc < 2 || !*argv) {
+		warnx(_("not enough arguments"));
+		errtryhelp(EXIT_FAILURE);
 	}
 
-	if (argc < 2)
-		usage(stderr);
-
 	prio = strtol(*argv, &endptr, 10);
-	if (*endptr)
-		usage(stderr);
-
+	if (*endptr) {
+		warnx(_("invalid priority '%s'"), *argv);
+		errtryhelp(EXIT_FAILURE);
+	}
 	argc--;
 	argv++;
 
@@ -187,7 +219,7 @@ int main(int argc, char **argv)
 				continue;
 			}
 		}
-		errs |= donice(which, who, prio);
+		errs |= donice(which, who, prio, relative);
 	}
 	return errs != 0 ? EXIT_FAILURE : EXIT_SUCCESS;
 }
